
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: BONUS_OBJECTIVE_TRACKER_MODULE, next, ObjectiveTrackerBlocksFrame, ObjectiveTrackerFrame, RegisterAttributeDriver, table, WORLD_QUEST_TRACKER_MODULE

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Blizzard ObjectiveTracker")

local OBJECTIVE_TRACKER_BUTTONS = {}
local OBJECTIVE_TRACKER_HEADERS = {}
local SHOW_STATE_TEXT = "show"

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	if BONUS_OBJECTIVE_TRACKER_MODULE and ObjectiveTrackerBlocksFrame and WORLD_QUEST_TRACKER_MODULE then
		for _, header in next, {
			ObjectiveTrackerFrame.HeaderMenu,
			BONUS_OBJECTIVE_TRACKER_MODULE.Header,
			ObjectiveTrackerBlocksFrame.AchievementHeader,
			ObjectiveTrackerBlocksFrame.CampaignQuestHeader,
			ObjectiveTrackerBlocksFrame.MonthlyActivitiesHeader,
			ObjectiveTrackerBlocksFrame.QuestHeader,
			ObjectiveTrackerBlocksFrame.ScenarioHeader,
			ObjectiveTrackerBlocksFrame.UIWidgetsHeader,
			WORLD_QUEST_TRACKER_MODULE.Header,
		} do
			if header then
				if header.isHeader then
					table.insert(OBJECTIVE_TRACKER_HEADERS, header)
				end

				if header.MinimizeButton then
					table.insert(OBJECTIVE_TRACKER_BUTTONS, header.MinimizeButton)
				end
			end
		end
	end
end

function M:OnEnable()
	self:ConfigureArtwork()
	self:ConfigureButtons()
	self:ConfigureFrame()
	self:ConfigureVisibility()
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE")
	self:RegisterMessage("SIXUI_PROFILE_UPDATE")
end

function M:ConfigureArtwork()
	for _, header in next, OBJECTIVE_TRACKER_HEADERS do
		if self.config.showArtwork then
			header:GetRegions():Show()
		else
			header:GetRegions():Hide()
		end
	end
end

function M:ConfigureButtons()
	local alpha = (self.config.showButtons or self.isClutterKeyDown) and 1 or 0

	for _, button in next, OBJECTIVE_TRACKER_BUTTONS do
		button:SetAlpha(alpha)
	end
end

function M:ConfigureFrame()
	ObjectiveTrackerFrame:SetFrameStrata("BACKGROUND")
end

function M:ConfigureVisibility()
	RegisterAttributeDriver(ObjectiveTrackerFrame, "state-visibility", self:GetStateText())
end

function M:GetStateText()
	return self.config.stateText and self.config.stateText ~= "" and A.FlavourStateText(self.config.stateText) or SHOW_STATE_TEXT
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	self.isClutterKeyDown = state
	self:ConfigureButtons()
end

function M:SIXUI_CONFIG_UPDATE(event, module, config, key, ...)
	if module == self:GetName() then
		if key == "showArtwork" then
			self:ConfigureArtwork()
		elseif key == "showButtons" then
			self:ConfigureButtons()
		elseif key == "stateText" then
			self:ConfigureVisibility()
		end
	end
end

function M:SIXUI_PROFILE_UPDATE(event)
	self:ConfigureArtwork()
	self:ConfigureButtons()
	self:ConfigureVisibility()
end
